//  VHS Pixel Panel Tester V1.2
//  https://talk.vanhack.ca/
//  https://vanhack.ca/wp/about/
//  Code to test LEDs on VHS LED panels
//  Needs these libraries
//  https://github.com/2dom/PxMatrix
//  https://github.com/FastLED/FastLED
//  Runs on a ESP8266 module
//  Only supports a single VHS LED panel
//  Runs at full brightness to stress LEDs
//  VHS panels are 1/4 scan and run on 5V (allow 1A per panel)
//  Connection info and details:
//  https://github.com/2dom/PxMatrix
//  https://www.instructables.com/RGB-LED-Matrix-With-an-ESP8266/
//  Some basic logging sent to serial port (115200 bps)

#include <PxMatrix.h>
#include <FastLED.h> 

#define Color_Delay 1000        // time each color displays
#define White_Delay 2000        // time white displays
#define PxMATRIX_MAX_HEIGHT 16
#define PxMATRIX_MAX_WIDTH 16
#define PxMATRIX_double_buffer true

#ifdef ESP8266

#include <Ticker.h>
Ticker display_ticker;

#define P_LAT 16      // ESP8266 pin assignments
#define P_A 5
#define P_B 4
#define P_C 15
#define P_D 12
#define P_E 0
#define P_OE 2

#endif


// This defines the 'on' time of the display is us. The larger this number,
// the brighter the display. If too large the ESP will crash
uint8_t display_draw_time = 30; //30-60 is usually fine

PxMATRIX display(16, 16, P_LAT, P_OE, P_A, P_B, P_C);


// Some standard colors
uint16_t myRED = display.color565(255, 0, 0);
uint16_t myGREEN = display.color565(0, 255, 0);
uint16_t myBLUE = display.color565(0, 0, 255);
uint16_t myWHITE = display.color565(255, 255, 255);
uint16_t myYELLOW = display.color565(255, 255, 0);
uint16_t myCYAN = display.color565(0, 255, 255);
uint16_t myMAGENTA = display.color565(255, 0, 255);
uint16_t myBLACK = display.color565(0, 0, 0);


#ifdef ESP8266
// ISR for display refresh
void display_updater()
{
  display.display(display_draw_time);
}
#endif


void display_update_enable(bool is_enable)
{

#ifdef ESP8266
  if (is_enable)
    display_ticker.attach(0.001, display_updater);
  else
    display_ticker.detach();
#endif

#ifdef ESP32
  if (is_enable)
  {
    timer = timerBegin(0, 80, true);
    timerAttachInterrupt(timer, &display_updater, true);
    timerAlarmWrite(timer, 2000, true);
    timerAlarmEnable(timer);
  }
  else
  {
    timerDetachInterrupt(timer);
    timerAlarmDisable(timer);
  }
#endif
}



void setup() {

  Serial.begin(115200);

  delay(100);

  Serial.println();
  Serial.println("VHS Pixel Panel Tester v1.2");

  // Define your display layout here, e.g. 1/4 step
  display.begin(4);

  // Set the multiplex pattern {LINE, ZIGZAG,ZZAGG, ZAGGIZ, WZAGZIG, VZAG, ZAGZIG} (default is LINE)
  display.setScanPattern(ZIGZAG);

  // Set the time in microseconds that we pause after selecting each mux channel
  // (May help if some rows are missing / the mux chip is too slow)
  display.setMuxDelay(1, 1, 1, 1, 1);

  // Set driver chip type
  display.setDriverChip(SHIFT);

  display_update_enable(true);
  delay(100);

}




void loop()
{

  Serial.println("Testing RED");
  display.fillScreen(myRED);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

  Serial.println("Testing GREEN");
  display.fillScreen(myGREEN);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

  Serial.println("Testing BLUE");
  display.fillScreen(myBLUE);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

    Serial.println("Testing YELLOW");
  display.fillScreen(myYELLOW);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

  Serial.println("Testing CYAN");
  display.fillScreen(myCYAN);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

  Serial.println("Testing MAGENTA");
  display.fillScreen(myMAGENTA);
  display.showBuffer();
  delay(Color_Delay);
  display.clearDisplay();
  display.showBuffer();

  Serial.println("Testing WHITE");
  display.fillScreen(myWHITE);
  display.showBuffer();
  delay(White_Delay);
  display.clearDisplay();
  display.showBuffer();

}
